@@ -7,19 +7,20 @@ import (
 	"os"
 	"path/filepath"
 	"runtime"
+	"slices"
 	"time"
 
-	"github.com/hashicorp/go-version"
-	"github.com/traefik/traefik/v2/pkg/log"
-	"github.com/traefik/traefik/v2/pkg/provider/kubernetes/k8s"
-	traefikversion "github.com/traefik/traefik/v2/pkg/version"
+	"github.com/rs/zerolog/log"
+	"github.com/traefik/traefik/v3/pkg/provider/kubernetes/k8s"
+	"github.com/traefik/traefik/v3/pkg/types"
+	traefikversion "github.com/traefik/traefik/v3/pkg/version"
 	corev1 "k8s.io/api/core/v1"
+	discoveryv1 "k8s.io/api/discovery/v1"
 	netv1 "k8s.io/api/networking/v1"
-	netv1beta1 "k8s.io/api/networking/v1beta1"
 	kerror "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/labels"
-	"k8s.io/apimachinery/pkg/util/intstr"
+	"k8s.io/apimachinery/pkg/selection"
 	kinformers "k8s.io/client-go/informers"
 	kclientset "k8s.io/client-go/kubernetes"
 	"k8s.io/client-go/rest"
@@ -40,21 +41,22 @@ type Client interface {
 	GetIngressClasses() ([]*netv1.IngressClass, error)
 	GetService(namespace, name string) (*corev1.Service, bool, error)
 	GetSecret(namespace, name string) (*corev1.Secret, bool, error)
-	GetEndpoints(namespace, name string) (*corev1.Endpoints, bool, error)
+	GetNodes() ([]*corev1.Node, bool, error)
+	GetEndpointSlicesForService(namespace, serviceName string) ([]*discoveryv1.EndpointSlice, error)
 	UpdateIngressStatus(ing *netv1.Ingress, ingStatus []netv1.IngressLoadBalancerIngress) error
-	GetServerVersion() *version.Version
 }
 
 type clientWrapper struct {
-	clientset            kclientset.Interface
-	factoriesKube        map[string]kinformers.SharedInformerFactory
-	factoriesSecret      map[string]kinformers.SharedInformerFactory
-	factoriesIngress     map[string]kinformers.SharedInformerFactory
-	clusterFactory       kinformers.SharedInformerFactory
-	ingressLabelSelector string
-	isNamespaceAll       bool
-	watchedNamespaces    []string
-	serverVersion        *version.Version
+	clientset                   kclientset.Interface
+	clusterScopeFactory         kinformers.SharedInformerFactory
+	factoriesKube               map[string]kinformers.SharedInformerFactory
+	factoriesSecret             map[string]kinformers.SharedInformerFactory
+	factoriesIngress            map[string]kinformers.SharedInformerFactory
+	ingressLabelSelector        string
+	isNamespaceAll              bool
+	disableIngressClassInformer bool // Deprecated.
+	disableClusterScopeInformer bool
+	watchedNamespaces           []string
 }
 
 // newInClusterClient returns a new Provider client that is expected to run
@@ -83,14 +85,19 @@ func newExternalClusterClientFromFile(file string) (*clientWrapper, error) {
 // newExternalClusterClient returns a new Provider client that may run outside
 // of the cluster.
 // The endpoint parameter must not be empty.
-func newExternalClusterClient(endpoint, token, caFilePath string) (*clientWrapper, error) {
+func newExternalClusterClient(endpoint, caFilePath string, token types.FileOrContent) (*clientWrapper, error) {
 	if endpoint == "" {
 		return nil, errors.New("endpoint missing for external cluster client")
 	}
 
+	tokenData, err := token.Read()
+	if err != nil {
+		return nil, fmt.Errorf("read token: %w", err)
+	}
+
 	config := &rest.Config{
 		Host:        endpoint,
-		BearerToken: token,
+		BearerToken: string(tokenData),
 	}
 
 	if caFilePath != "" {
@@ -132,19 +139,6 @@ func newClientImpl(clientset kclientset.Interface) *clientWrapper {
 
 // WatchAll starts namespace-specific controllers for all relevant kinds.
 func (c *clientWrapper) WatchAll(namespaces []string, stopCh <-chan struct{}) (<-chan interface{}, error) {
-	// Get and store the serverVersion for future use.
-	serverVersionInfo, err := c.clientset.Discovery().ServerVersion()
-	if err != nil {
-		return nil, fmt.Errorf("could not retrieve server version: %w", err)
-	}
-
-	serverVersion, err := version.NewVersion(serverVersionInfo.GitVersion)
-	if err != nil {
-		return nil, fmt.Errorf("could not parse server version: %w", err)
-	}
-
-	c.serverVersion = serverVersion
-
 	eventCh := make(chan interface{}, 1)
 	eventHandler := &k8s.ResourceEventHandler{Ev: eventCh}
 
@@ -166,16 +160,9 @@ func (c *clientWrapper) WatchAll(namespaces []string, stopCh <-chan struct{}) (<
 	for _, ns := range namespaces {
 		factoryIngress := kinformers.NewSharedInformerFactoryWithOptions(c.clientset, resyncPeriod, kinformers.WithNamespace(ns), kinformers.WithTweakListOptions(matchesLabelSelector))
 
-		if supportsNetworkingV1Ingress(serverVersion) {
-			_, err = factoryIngress.Networking().V1().Ingresses().Informer().AddEventHandler(eventHandler)
-			if err != nil {
-				return nil, err
-			}
-		} else {
-			_, err = factoryIngress.Networking().V1beta1().Ingresses().Informer().AddEventHandler(eventHandler)
-			if err != nil {
-				return nil, err
-			}
+		_, err := factoryIngress.Networking().V1().Ingresses().Informer().AddEventHandler(eventHandler)
+		if err != nil {
+			return nil, err
 		}
 
 		c.factoriesIngress[ns] = factoryIngress
@@ -185,7 +172,7 @@ func (c *clientWrapper) WatchAll(namespaces []string, stopCh <-chan struct{}) (<
 		if err != nil {
 			return nil, err
 		}
-		_, err = factoryKube.Core().V1().Endpoints().Informer().AddEventHandler(eventHandler)
+		_, err = factoryKube.Discovery().V1().EndpointSlices().Informer().AddEventHandler(eventHandler)
 		if err != nil {
 			return nil, err
 		}
@@ -206,45 +193,45 @@ func (c *clientWrapper) WatchAll(namespaces []string, stopCh <-chan struct{}) (<
 	}
 
 	for _, ns := range namespaces {
-		for typ, ok := range c.factoriesIngress[ns].WaitForCacheSync(stopCh) {
+		for t, ok := range c.factoriesIngress[ns].WaitForCacheSync(stopCh) {
 			if !ok {
-				return nil, fmt.Errorf("timed out waiting for controller caches to sync %s in namespace %q", typ, ns)
+				return nil, fmt.Errorf("timed out waiting for controller caches to sync %s in namespace %q", t.String(), ns)
 			}
 		}
 
-		for typ, ok := range c.factoriesKube[ns].WaitForCacheSync(stopCh) {
+		for t, ok := range c.factoriesKube[ns].WaitForCacheSync(stopCh) {
 			if !ok {
-				return nil, fmt.Errorf("timed out waiting for controller caches to sync %s in namespace %q", typ, ns)
+				return nil, fmt.Errorf("timed out waiting for controller caches to sync %s in namespace %q", t.String(), ns)
 			}
 		}
 
-		for typ, ok := range c.factoriesSecret[ns].WaitForCacheSync(stopCh) {
+		for t, ok := range c.factoriesSecret[ns].WaitForCacheSync(stopCh) {
 			if !ok {
-				return nil, fmt.Errorf("timed out waiting for controller caches to sync %s in namespace %q", typ, ns)
+				return nil, fmt.Errorf("timed out waiting for controller caches to sync %s in namespace %q", t.String(), ns)
 			}
 		}
 	}
 
-	if supportsIngressClass(serverVersion) {
-		c.clusterFactory = kinformers.NewSharedInformerFactoryWithOptions(c.clientset, resyncPeriod)
+	if !c.disableIngressClassInformer || !c.disableClusterScopeInformer {
+		c.clusterScopeFactory = kinformers.NewSharedInformerFactory(c.clientset, resyncPeriod)
 
-		if supportsNetworkingV1Ingress(serverVersion) {
-			_, err = c.clusterFactory.Networking().V1().IngressClasses().Informer().AddEventHandler(eventHandler)
-			if err != nil {
-				return nil, err
-			}
-		} else {
-			_, err = c.clusterFactory.Networking().V1beta1().IngressClasses().Informer().AddEventHandler(eventHandler)
+		_, err := c.clusterScopeFactory.Networking().V1().IngressClasses().Informer().AddEventHandler(eventHandler)
+		if err != nil {
+			return nil, err
+		}
+
+		if !c.disableClusterScopeInformer {
+			_, err = c.clusterScopeFactory.Core().V1().Nodes().Informer().AddEventHandler(eventHandler)
 			if err != nil {
 				return nil, err
 			}
 		}
 
-		c.clusterFactory.Start(stopCh)
+		c.clusterScopeFactory.Start(stopCh)
 
-		for typ, ok := range c.clusterFactory.WaitForCacheSync(stopCh) {
+		for t, ok := range c.clusterScopeFactory.WaitForCacheSync(stopCh) {
 			if !ok {
-				return nil, fmt.Errorf("timed out waiting for controller caches to sync %s", typ)
+				return nil, fmt.Errorf("timed out waiting for controller caches to sync %s", t.String())
 			}
 		}
 	}
@@ -256,107 +243,35 @@ func (c *clientWrapper) WatchAll(namespaces []string, stopCh <-chan struct{}) (<
 func (c *clientWrapper) GetIngresses() []*netv1.Ingress {
 	var results []*netv1.Ingress
 
-	isNetworkingV1Supported := supportsNetworkingV1Ingress(c.serverVersion)
-
 	for ns, factory := range c.factoriesIngress {
-		if isNetworkingV1Supported {
-			// networking
-			listNew, err := factory.Networking().V1().Ingresses().Lister().List(labels.Everything())
-			if err != nil {
-				log.WithoutContext().Errorf("Failed to list ingresses in namespace %s: %v", ns, err)
-				continue
-			}
-
-			results = append(results, listNew...)
-			continue
-		}
-
-		// networking beta
-		list, err := factory.Networking().V1beta1().Ingresses().Lister().List(labels.Everything())
+		// networking
+		listNew, err := factory.Networking().V1().Ingresses().Lister().List(labels.Everything())
 		if err != nil {
-			log.WithoutContext().Errorf("Failed to list ingresses in namespace %s: %v", ns, err)
+			log.Error().Err(err).Msgf("Failed to list ingresses in namespace %s", ns)
 			continue
 		}
 
-		for _, ing := range list {
-			n, err := convert[netv1.Ingress](ing)
-			if err != nil {
-				log.WithoutContext().Errorf("Failed to convert ingress %s from networking/v1beta1 to networking/v1: %v", ns, err)
-				continue
-			}
-
-			addServiceFromV1Beta1(n, *ing)
-
-			results = append(results, n)
-		}
+		results = append(results, listNew...)
 	}
+
 	return results
 }
 
-func addServiceFromV1Beta1(ing *netv1.Ingress, old netv1beta1.Ingress) {
-	if old.Spec.Backend != nil {
-		port := netv1.ServiceBackendPort{}
-		if old.Spec.Backend.ServicePort.Type == intstr.Int {
-			port.Number = old.Spec.Backend.ServicePort.IntVal
-		} else {
-			port.Name = old.Spec.Backend.ServicePort.StrVal
-		}
-
-		if old.Spec.Backend.ServiceName != "" {
-			ing.Spec.DefaultBackend = &netv1.IngressBackend{
-				Service: &netv1.IngressServiceBackend{
-					Name: old.Spec.Backend.ServiceName,
-					Port: port,
-				},
-			}
-		}
-	}
-
-	for rc, rule := range ing.Spec.Rules {
-		if rule.HTTP == nil {
-			continue
-		}
-		for pc, path := range rule.HTTP.Paths {
-			if path.Backend.Service == nil {
-				oldBackend := old.Spec.Rules[rc].HTTP.Paths[pc].Backend
-
-				port := netv1.ServiceBackendPort{}
-				if oldBackend.ServicePort.Type == intstr.Int {
-					port.Number = oldBackend.ServicePort.IntVal
-				} else {
-					port.Name = oldBackend.ServicePort.StrVal
-				}
-
-				svc := netv1.IngressServiceBackend{
-					Name: oldBackend.ServiceName,
-					Port: port,
-				}
-
-				ing.Spec.Rules[rc].HTTP.Paths[pc].Backend.Service = &svc
-			}
-		}
-	}
-}
-
 // UpdateIngressStatus updates an Ingress with a provided status.
 func (c *clientWrapper) UpdateIngressStatus(src *netv1.Ingress, ingStatus []netv1.IngressLoadBalancerIngress) error {
 	if !c.isWatchedNamespace(src.Namespace) {
 		return fmt.Errorf("failed to get ingress %s/%s: namespace is not within watched namespaces", src.Namespace, src.Name)
 	}
 
-	if !supportsNetworkingV1Ingress(c.serverVersion) {
-		return c.updateIngressStatusOld(src, ingStatus)
-	}
-
 	ing, err := c.factoriesIngress[c.lookupNamespace(src.Namespace)].Networking().V1().Ingresses().Lister().Ingresses(src.Namespace).Get(src.Name)
 	if err != nil {
 		return fmt.Errorf("failed to get ingress %s/%s: %w", src.Namespace, src.Name, err)
 	}
 
-	logger := log.WithoutContext().WithField("namespace", ing.Namespace).WithField("ingress", ing.Name)
+	logger := log.With().Str("namespace", ing.Namespace).Str("ingress", ing.Name).Logger()
 
 	if isLoadBalancerIngressEquals(ing.Status.LoadBalancer.Ingress, ingStatus) {
-		logger.Debug("Skipping ingress status update")
+		logger.Debug().Msg("Skipping ingress status update")
 		return nil
 	}
 
@@ -371,44 +286,7 @@ func (c *clientWrapper) UpdateIngressStatus(src *netv1.Ingress, ingStatus []netv
 		return fmt.Errorf("failed to update ingress status %s/%s: %w", src.Namespace, src.Name, err)
 	}
 
-	logger.Info("Updated ingress status")
-	return nil
-}
-
-func (c *clientWrapper) updateIngressStatusOld(src *netv1.Ingress, ingStatus []netv1.IngressLoadBalancerIngress) error {
-	ing, err := c.factoriesIngress[c.lookupNamespace(src.Namespace)].Networking().V1beta1().Ingresses().Lister().Ingresses(src.Namespace).Get(src.Name)
-	if err != nil {
-		return fmt.Errorf("failed to get ingress %s/%s: %w", src.Namespace, src.Name, err)
-	}
-
-	logger := log.WithoutContext().WithField("namespace", ing.Namespace).WithField("ingress", ing.Name)
-
-	ingresses, err := convertSlice[netv1.IngressLoadBalancerIngress](ing.Status.LoadBalancer.Ingress)
-	if err != nil {
-		return err
-	}
-
-	if isLoadBalancerIngressEquals(ingresses, ingStatus) {
-		logger.Debug("Skipping ingress status update")
-		return nil
-	}
-
-	ingressesBeta1, err := convertSlice[netv1beta1.IngressLoadBalancerIngress](ingStatus)
-	if err != nil {
-		return err
-	}
-
-	ingCopy := ing.DeepCopy()
-	ingCopy.Status = netv1beta1.IngressStatus{LoadBalancer: netv1beta1.IngressLoadBalancerStatus{Ingress: ingressesBeta1}}
-
-	ctx, cancel := context.WithTimeout(context.Background(), defaultTimeout)
-	defer cancel()
-
-	_, err = c.clientset.NetworkingV1beta1().Ingresses(ingCopy.Namespace).UpdateStatus(ctx, ingCopy, metav1.UpdateOptions{})
-	if err != nil {
-		return fmt.Errorf("failed to update ingress status %s/%s: %w", src.Namespace, src.Name, err)
-	}
-	logger.Info("Updated ingress status")
+	logger.Info().Msg("Updated ingress status")
 	return nil
 }
 
@@ -443,15 +321,20 @@ func (c *clientWrapper) GetService(namespace, name string) (*corev1.Service, boo
 	return service, exist, err
 }
 
-// GetEndpoints returns the named endpoints from the given namespace.
-func (c *clientWrapper) GetEndpoints(namespace, name string) (*corev1.Endpoints, bool, error) {
+// GetEndpointSlicesForService returns the EndpointSlices for the given service name in the given namespace.
+func (c *clientWrapper) GetEndpointSlicesForService(namespace, serviceName string) ([]*discoveryv1.EndpointSlice, error) {
 	if !c.isWatchedNamespace(namespace) {
-		return nil, false, fmt.Errorf("failed to get endpoints %s/%s: namespace is not within watched namespaces", namespace, name)
+		return nil, fmt.Errorf("failed to get endpointslices for service %s/%s: namespace is not within watched namespaces", namespace, serviceName)
 	}
 
-	endpoint, err := c.factoriesKube[c.lookupNamespace(namespace)].Core().V1().Endpoints().Lister().Endpoints(namespace).Get(name)
-	exist, err := translateNotFoundError(err)
-	return endpoint, exist, err
+	serviceLabelRequirement, err := labels.NewRequirement(discoveryv1.LabelServiceName, selection.Equals, []string{serviceName})
+	if err != nil {
+		return nil, fmt.Errorf("failed to create service label selector requirement: %w", err)
+	}
+	serviceSelector := labels.NewSelector()
+	serviceSelector = serviceSelector.Add(*serviceLabelRequirement)
+
+	return c.factoriesKube[c.lookupNamespace(namespace)].Discovery().V1().EndpointSlices().Lister().EndpointSlices(namespace).List(serviceSelector)
 }
 
 // GetSecret returns the named secret from the given namespace.
@@ -465,33 +348,19 @@ func (c *clientWrapper) GetSecret(namespace, name string) (*corev1.Secret, bool,
 	return secret, exist, err
 }
 
+func (c *clientWrapper) GetNodes() ([]*corev1.Node, bool, error) {
+	nodes, err := c.clusterScopeFactory.Core().V1().Nodes().Lister().List(labels.Everything())
+	exist, err := translateNotFoundError(err)
+	return nodes, exist, err
+}
+
 func (c *clientWrapper) GetIngressClasses() ([]*netv1.IngressClass, error) {
-	if c.clusterFactory == nil {
+	if c.clusterScopeFactory == nil {
 		return nil, errors.New("cluster factory not loaded")
 	}
 
 	var ics []*netv1.IngressClass
-	if !supportsNetworkingV1Ingress(c.serverVersion) {
-		ingressClasses, err := c.clusterFactory.Networking().V1beta1().IngressClasses().Lister().List(labels.Everything())
-		if err != nil {
-			return nil, err
-		}
-
-		for _, ic := range ingressClasses {
-			if ic.Spec.Controller == traefikDefaultIngressClassController {
-				icN, err := convert[netv1.IngressClass](ic)
-				if err != nil {
-					log.WithoutContext().Errorf("Failed to convert ingress class %s from networking/v1beta1 to networking/v1: %v", ic.Name, err)
-					continue
-				}
-				ics = append(ics, icN)
-			}
-		}
-
-		return ics, nil
-	}
-
-	ingressClasses, err := c.clusterFactory.Networking().V1().IngressClasses().Lister().List(labels.Everything())
+	ingressClasses, err := c.clusterScopeFactory.Networking().V1().IngressClasses().Lister().List(labels.Everything())
 	if err != nil {
 		return nil, err
 	}
@@ -518,11 +387,6 @@ func (c *clientWrapper) lookupNamespace(ns string) string {
 	return ns
 }
 
-// GetServerVersion returns the cluster server version, or an error.
-func (c *clientWrapper) GetServerVersion() *version.Version {
-	return c.serverVersion
-}
-
 // translateNotFoundError will translate a "not found" error to a boolean return
 // value which indicates if the resource exists and a nil error.
 func translateNotFoundError(err error) (bool, error) {
@@ -538,20 +402,8 @@ func (c *clientWrapper) isWatchedNamespace(ns string) bool {
 	if c.isNamespaceAll {
 		return true
 	}
-	for _, watchedNamespace := range c.watchedNamespaces {
-		if watchedNamespace == ns {
-			return true
-		}
-	}
-	return false
-}
 
-// IngressClass objects are supported since Kubernetes v1.18.
-// See https://kubernetes.io/docs/concepts/services-networking/ingress/#ingress-class
-func supportsIngressClass(serverVersion *version.Version) bool {
-	ingressClassVersion := version.Must(version.NewVersion("1.18"))
-
-	return ingressClassVersion.LessThanOrEqual(serverVersion)
+	return slices.Contains(c.watchedNamespaces, ns)
 }
 
 // filterIngressClassByName return a slice containing ingressclasses with the correct name.
@@ -566,11 +418,3 @@ func filterIngressClassByName(ingressClassName string, ics []*netv1.IngressClass
 
 	return ingressClasses
 }
-
-// Ingress in networking.k8s.io/v1 is supported starting 1.19.
-// thus, we query it in K8s starting 1.19.
-func supportsNetworkingV1Ingress(serverVersion *version.Version) bool {
-	ingressNetworkingVersion := version.Must(version.NewVersion("1.19"))
-
-	return serverVersion.GreaterThanOrEqual(ingressNetworkingVersion)
-}
