@@ -10,35 +10,28 @@ import (
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	ptypes "github.com/traefik/paerser/types"
-	"github.com/traefik/traefik/v2/pkg/config/dynamic"
-	"github.com/traefik/traefik/v2/pkg/config/static"
-	"github.com/traefik/traefik/v2/pkg/ping"
-	"github.com/traefik/traefik/v2/pkg/plugins"
-	"github.com/traefik/traefik/v2/pkg/provider/acme"
-	"github.com/traefik/traefik/v2/pkg/provider/consulcatalog"
-	"github.com/traefik/traefik/v2/pkg/provider/docker"
-	"github.com/traefik/traefik/v2/pkg/provider/ecs"
-	"github.com/traefik/traefik/v2/pkg/provider/file"
-	"github.com/traefik/traefik/v2/pkg/provider/http"
-	"github.com/traefik/traefik/v2/pkg/provider/kubernetes/crd"
-	"github.com/traefik/traefik/v2/pkg/provider/kubernetes/gateway"
-	"github.com/traefik/traefik/v2/pkg/provider/kubernetes/ingress"
-	"github.com/traefik/traefik/v2/pkg/provider/kv"
-	"github.com/traefik/traefik/v2/pkg/provider/kv/consul"
-	"github.com/traefik/traefik/v2/pkg/provider/kv/etcd"
-	"github.com/traefik/traefik/v2/pkg/provider/kv/redis"
-	"github.com/traefik/traefik/v2/pkg/provider/kv/zk"
-	"github.com/traefik/traefik/v2/pkg/provider/marathon"
-	"github.com/traefik/traefik/v2/pkg/provider/rancher"
-	"github.com/traefik/traefik/v2/pkg/provider/rest"
-	traefiktls "github.com/traefik/traefik/v2/pkg/tls"
-	"github.com/traefik/traefik/v2/pkg/tracing/datadog"
-	"github.com/traefik/traefik/v2/pkg/tracing/elastic"
-	"github.com/traefik/traefik/v2/pkg/tracing/haystack"
-	"github.com/traefik/traefik/v2/pkg/tracing/instana"
-	"github.com/traefik/traefik/v2/pkg/tracing/jaeger"
-	"github.com/traefik/traefik/v2/pkg/tracing/zipkin"
-	"github.com/traefik/traefik/v2/pkg/types"
+	"github.com/traefik/traefik/v3/pkg/config/dynamic"
+	"github.com/traefik/traefik/v3/pkg/config/static"
+	otypes "github.com/traefik/traefik/v3/pkg/observability/types"
+	"github.com/traefik/traefik/v3/pkg/ping"
+	"github.com/traefik/traefik/v3/pkg/plugins"
+	"github.com/traefik/traefik/v3/pkg/provider/acme"
+	"github.com/traefik/traefik/v3/pkg/provider/consulcatalog"
+	"github.com/traefik/traefik/v3/pkg/provider/docker"
+	"github.com/traefik/traefik/v3/pkg/provider/ecs"
+	"github.com/traefik/traefik/v3/pkg/provider/file"
+	"github.com/traefik/traefik/v3/pkg/provider/http"
+	"github.com/traefik/traefik/v3/pkg/provider/kubernetes/crd"
+	"github.com/traefik/traefik/v3/pkg/provider/kubernetes/gateway"
+	"github.com/traefik/traefik/v3/pkg/provider/kubernetes/ingress"
+	"github.com/traefik/traefik/v3/pkg/provider/kv"
+	"github.com/traefik/traefik/v3/pkg/provider/kv/consul"
+	"github.com/traefik/traefik/v3/pkg/provider/kv/etcd"
+	"github.com/traefik/traefik/v3/pkg/provider/kv/redis"
+	"github.com/traefik/traefik/v3/pkg/provider/kv/zk"
+	"github.com/traefik/traefik/v3/pkg/provider/rest"
+	traefiktls "github.com/traefik/traefik/v3/pkg/tls"
+	"github.com/traefik/traefik/v3/pkg/types"
 )
 
 var updateExpected = flag.Bool("update_expected", false, "Update expected files in fixtures")
@@ -65,6 +58,11 @@ func init() {
 						},
 					},
 				},
+				Observability: &dynamic.RouterObservabilityConfig{
+					AccessLogs: pointer(true),
+					Tracing:    pointer(true),
+					Metrics:    pointer(true),
+				},
 			},
 		},
 		Services: map[string]*dynamic.Service{
@@ -82,17 +80,17 @@ func init() {
 						Scheme:          "foo",
 						Path:            "foo",
 						Port:            42,
-						Interval:        "foo",
-						Timeout:         "foo",
+						Interval:        ptypes.Duration(111 * time.Second),
+						Timeout:         ptypes.Duration(111 * time.Second),
 						Hostname:        "foo",
-						FollowRedirects: boolPtr(true),
+						FollowRedirects: pointer(true),
 						Headers: map[string]string{
 							"foo": "bar",
 						},
 					},
-					PassHostHeader: boolPtr(true),
+					PassHostHeader: pointer(true),
 					ResponseForwarding: &dynamic.ResponseForwarding{
-						FlushInterval: "foo",
+						FlushInterval: ptypes.Duration(111 * time.Second),
 					},
 					ServersTransport: "foo",
 					Servers: []dynamic.Server{
@@ -107,7 +105,7 @@ func init() {
 					Services: []dynamic.WRRService{
 						{
 							Name:   "foo",
-							Weight: intPtr(42),
+							Weight: pointer(42),
 						},
 					},
 					Sticky: &dynamic.Sticky{
@@ -123,7 +121,7 @@ func init() {
 			"baz": {
 				Mirroring: &dynamic.Mirroring{
 					Service:     "foo",
-					MaxBodySize: int64Ptr(42),
+					MaxBodySize: pointer[int64](42),
 					Mirrors: []dynamic.MirrorService{
 						{
 							Name:    "foo",
@@ -137,7 +135,7 @@ func init() {
 			"foo": {
 				ServerName:         "foo",
 				InsecureSkipVerify: true,
-				RootCAs:            []traefiktls.FileOrContent{"rootca.pem"},
+				RootCAs:            []types.FileOrContent{"rootca.pem"},
 				Certificates: []traefiktls.Certificate{
 					{
 						CertFile: "cert.pem",
@@ -175,8 +173,7 @@ func init() {
 					Prefix: "foo",
 				},
 				StripPrefix: &dynamic.StripPrefix{
-					Prefixes:   []string{"foo"},
-					ForceSlash: true,
+					Prefixes: []string{"foo"},
 				},
 				StripPrefixRegex: &dynamic.StripPrefixRegex{
 					Regex: []string{"foo"},
@@ -191,7 +188,7 @@ func init() {
 				Chain: &dynamic.Chain{
 					Middlewares: []string{"foo"},
 				},
-				IPWhiteList: &dynamic.IPWhiteList{
+				IPAllowList: &dynamic.IPAllowList{
 					SourceRange: []string{"foo"},
 					IPStrategy: &dynamic.IPStrategy{
 						Depth:       42,
@@ -211,11 +208,7 @@ func init() {
 					AddVaryHeader:                     true,
 					AllowedHosts:                      []string{"foo"},
 					HostsProxyHeaders:                 []string{"foo"},
-					SSLRedirect:                       true,
-					SSLTemporaryRedirect:              true,
-					SSLHost:                           "foo",
 					SSLProxyHeaders:                   map[string]string{"foo": "bar"},
-					SSLForceHost:                      true,
 					STSSeconds:                        42,
 					STSIncludeSubdomains:              true,
 					STSPreload:                        true,
@@ -226,9 +219,9 @@ func init() {
 					BrowserXSSFilter:                  true,
 					CustomBrowserXSSValue:             "foo",
 					ContentSecurityPolicy:             "foo",
+					ContentSecurityPolicyReportOnly:   "foo",
 					PublicKey:                         "foo",
 					ReferrerPolicy:                    "foo",
-					FeaturePolicy:                     "foo",
 					PermissionsPolicy:                 "foo",
 					IsDevelopment:                     true,
 				},
@@ -276,9 +269,8 @@ func init() {
 				},
 				ForwardAuth: &dynamic.ForwardAuth{
 					Address: "127.0.0.1",
-					TLS: &types.ClientTLS{
+					TLS: &dynamic.ClientTLS{
 						CA:                 "ca.pem",
-						CAOptional:         true,
 						Cert:               "cert.pem",
 						Key:                "cert.pem",
 						InsecureSkipVerify: true,
@@ -344,9 +336,7 @@ func init() {
 					Attempts:        42,
 					InitialInterval: 42,
 				},
-				ContentType: &dynamic.ContentType{
-					AutoDetect: true,
-				},
+				ContentType: &dynamic.ContentType{},
 				Plugin: map[string]dynamic.PluginConf{
 					"foo": {
 						"answer": struct{ Answer int }{
@@ -379,7 +369,6 @@ func init() {
 		Services: map[string]*dynamic.TCPService{
 			"foo": {
 				LoadBalancer: &dynamic.TCPServersLoadBalancer{
-					TerminationDelay: intPtr(42),
 					ProxyProtocol: &dynamic.ProxyProtocol{
 						Version: 42,
 					},
@@ -388,6 +377,7 @@ func init() {
 							Address: "127.0.0.1:8080",
 						},
 					},
+					ServersTransport: "foo",
 				},
 			},
 			"bar": {
@@ -395,12 +385,30 @@ func init() {
 					Services: []dynamic.TCPWRRService{
 						{
 							Name:   "foo",
-							Weight: intPtr(42),
+							Weight: pointer(42),
 						},
 					},
 				},
 			},
 		},
+		ServersTransports: map[string]*dynamic.TCPServersTransport{
+			"foo": {
+				TLS: &dynamic.TLSClientConfig{
+					ServerName:         "foo",
+					InsecureSkipVerify: true,
+					RootCAs:            []types.FileOrContent{"rootca.pem"},
+					Certificates: []traefiktls.Certificate{
+						{
+							CertFile: "cert.pem",
+							KeyFile:  "key.pem",
+						},
+					},
+				},
+				DialTimeout:      42,
+				DialKeepAlive:    42,
+				TerminationDelay: 42,
+			},
+		},
 	}
 	config.UDP = &dynamic.UDPConfiguration{
 		Routers: map[string]*dynamic.UDPRouter{
@@ -424,7 +432,7 @@ func init() {
 					Services: []dynamic.UDPWRRService{
 						{
 							Name:   "foo",
-							Weight: intPtr(42),
+							Weight: pointer(42),
 						},
 					},
 				},
@@ -439,7 +447,7 @@ func init() {
 				CipherSuites:     []string{"foo"},
 				CurvePreferences: []string{"foo"},
 				ClientAuth: traefiktls.ClientAuth{
-					CAFiles:        []traefiktls.FileOrContent{"ca.pem"},
+					CAFiles:        []types.FileOrContent{"ca.pem"},
 					ClientAuthType: "RequireAndVerifyClientCert",
 				},
 				SniStrict: true,
@@ -481,7 +489,7 @@ func TestAnonymize_dynamicConfiguration(t *testing.T) {
 	}
 
 	expected := strings.TrimSuffix(string(expectedConfiguration), "\n")
-	assert.Equal(t, expected, cleanJSON)
+	assert.JSONEq(t, expected, cleanJSON)
 }
 
 func TestSecure_dynamicConfiguration(t *testing.T) {
@@ -498,7 +506,7 @@ func TestSecure_dynamicConfiguration(t *testing.T) {
 	}
 
 	expected := strings.TrimSuffix(string(expectedConfiguration), "\n")
-	assert.Equal(t, expected, cleanJSON)
+	assert.JSONEq(t, expected, cleanJSON)
 }
 
 func TestDo_staticConfiguration(t *testing.T) {
@@ -511,7 +519,7 @@ func TestDo_staticConfiguration(t *testing.T) {
 
 	config.ServersTransport = &static.ServersTransport{
 		InsecureSkipVerify:  true,
-		RootCAs:             []traefiktls.FileOrContent{"root.ca"},
+		RootCAs:             []types.FileOrContent{"root.ca"},
 		MaxIdleConnsPerHost: 42,
 		ForwardingTimeouts: &static.ForwardingTimeouts{
 			DialTimeout:           42,
@@ -521,7 +529,7 @@ func TestDo_staticConfiguration(t *testing.T) {
 	}
 
 	config.EntryPoints = static.EntryPoints{
-		"foobar": {
+		"foobar": &static.EntryPoint{
 			Address: "foo Address",
 			Transport: &static.EntryPointsTransport{
 				LifeCycle: &static.LifeCycle{
@@ -569,7 +577,7 @@ func TestDo_staticConfiguration(t *testing.T) {
 
 	config.ServersTransport = &static.ServersTransport{
 		InsecureSkipVerify:  true,
-		RootCAs:             []traefiktls.FileOrContent{"RootCAs 1", "RootCAs 2", "RootCAs 3"},
+		RootCAs:             []types.FileOrContent{"RootCAs 1", "RootCAs 2", "RootCAs 3"},
 		MaxIdleConnsPerHost: 111,
 		ForwardingTimeouts: &static.ForwardingTimeouts{
 			DialTimeout:           ptypes.Duration(111 * time.Second),
@@ -578,6 +586,15 @@ func TestDo_staticConfiguration(t *testing.T) {
 		},
 	}
 
+	config.TCPServersTransport = &static.TCPServersTransport{
+		DialTimeout:   ptypes.Duration(111 * time.Second),
+		DialKeepAlive: ptypes.Duration(111 * time.Second),
+		TLS: &static.TLSClientConfig{
+			InsecureSkipVerify: true,
+			RootCAs:            []types.FileOrContent{"RootCAs 1", "RootCAs 2", "RootCAs 3"},
+		},
+	}
+
 	config.Providers.File = &file.Provider{
 		Directory:                 "file Directory",
 		Watch:                     true,
@@ -586,50 +603,46 @@ func TestDo_staticConfiguration(t *testing.T) {
 	}
 
 	config.Providers.Docker = &docker.Provider{
-		Constraints: `Label("foo", "bar")`,
-		Watch:       true,
-		Endpoint:    "MyEndPoint",
-		DefaultRule: "PathPrefix(`/`)",
-		TLS: &types.ClientTLS{
-			CA:                 "myCa",
-			CAOptional:         true,
-			Cert:               "mycert.pem",
-			Key:                "mycert.key",
-			InsecureSkipVerify: true,
+		Shared: docker.Shared{
+			ExposedByDefault:   true,
+			Constraints:        `Label("foo", "bar")`,
+			AllowEmptyServices: true,
+			Network:            "MyNetwork",
+			UseBindPortIP:      true,
+			Watch:              true,
+			DefaultRule:        "PathPrefix(`/`)",
+		},
+		ClientConfig: docker.ClientConfig{
+			Endpoint: "MyEndPoint", TLS: &types.ClientTLS{
+				CA:                 "myCa",
+				Cert:               "mycert.pem",
+				Key:                "mycert.key",
+				InsecureSkipVerify: true,
+			},
+			HTTPClientTimeout: 42,
 		},
-		ExposedByDefault:        true,
-		UseBindPortIP:           true,
-		SwarmMode:               true,
-		Network:                 "MyNetwork",
-		SwarmModeRefreshSeconds: 42,
-		HTTPClientTimeout:       42,
 	}
 
-	config.Providers.Marathon = &marathon.Provider{
-		Constraints:      `Label("foo", "bar")`,
-		Trace:            true,
-		Watch:            true,
-		Endpoint:         "foobar",
-		DefaultRule:      "PathPrefix(`/`)",
-		ExposedByDefault: true,
-		DCOSToken:        "foobar",
-		TLS: &types.ClientTLS{
-			CA:                 "myCa",
-			CAOptional:         true,
-			Cert:               "mycert.pem",
-			Key:                "mycert.key",
-			InsecureSkipVerify: true,
+	config.Providers.Swarm = &docker.SwarmProvider{
+		Shared: docker.Shared{
+			ExposedByDefault:   true,
+			Constraints:        `Label("foo", "bar")`,
+			AllowEmptyServices: true,
+			Network:            "MyNetwork",
+			UseBindPortIP:      true,
+			Watch:              true,
+			DefaultRule:        "PathPrefix(`/`)",
 		},
-		DialerTimeout:         42,
-		ResponseHeaderTimeout: 42,
-		TLSHandshakeTimeout:   42,
-		KeepAlive:             42,
-		ForceTaskHostname:     true,
-		Basic: &marathon.Basic{
-			HTTPBasicAuthUser: "user",
-			HTTPBasicPassword: "password",
+		ClientConfig: docker.ClientConfig{
+			Endpoint: "MyEndPoint", TLS: &types.ClientTLS{
+				CA:                 "myCa",
+				Cert:               "mycert.pem",
+				Key:                "mycert.key",
+				InsecureSkipVerify: true,
+			},
+			HTTPClientTimeout: 42,
 		},
-		RespectReadinessChecks: true,
+		RefreshSeconds: 42,
 	}
 
 	config.Providers.KubernetesIngress = &ingress.Provider{
@@ -670,17 +683,6 @@ func TestDo_staticConfiguration(t *testing.T) {
 		Insecure: true,
 	}
 
-	config.Providers.Rancher = &rancher.Provider{
-		Constraints:               `Label("foo", "bar")`,
-		Watch:                     true,
-		DefaultRule:               "PathPrefix(`/`)",
-		ExposedByDefault:          true,
-		EnableServiceHealthFilter: true,
-		RefreshSeconds:            42,
-		IntervalPoll:              true,
-		Prefix:                    "MyPrefix",
-	}
-
 	config.Providers.ConsulCatalog = &consulcatalog.ProviderBuilder{
 		Configuration: consulcatalog.Configuration{
 			Constraints: `Label("foo", "bar")`,
@@ -691,7 +693,6 @@ func TestDo_staticConfiguration(t *testing.T) {
 				Token:      "MyToken",
 				TLS: &types.ClientTLS{
 					CA:                 "myCa",
-					CAOptional:         true,
 					Cert:               "mycert.pem",
 					Key:                "mycert.key",
 					InsecureSkipVerify: true,
@@ -710,7 +711,6 @@ func TestDo_staticConfiguration(t *testing.T) {
 			ExposedByDefault:  true,
 			DefaultRule:       "PathPrefix(`/`)",
 		},
-		Namespace:  "ns",
 		Namespaces: []string{"ns1", "ns2"},
 	}
 
@@ -735,12 +735,10 @@ func TestDo_staticConfiguration(t *testing.T) {
 		Token: "secret",
 		TLS: &types.ClientTLS{
 			CA:                 "myCa",
-			CAOptional:         true,
 			Cert:               "mycert.pem",
 			Key:                "mycert.key",
 			InsecureSkipVerify: true,
 		},
-		Namespace:  "ns",
 		Namespaces: []string{"ns1", "ns2"},
 	}
 
@@ -753,7 +751,6 @@ func TestDo_staticConfiguration(t *testing.T) {
 		Password: "password",
 		TLS: &types.ClientTLS{
 			CA:                 "myCa",
-			CAOptional:         true,
 			Cert:               "mycert.pem",
 			Key:                "mycert.key",
 			InsecureSkipVerify: true,
@@ -778,7 +775,6 @@ func TestDo_staticConfiguration(t *testing.T) {
 		Password: "password",
 		TLS: &types.ClientTLS{
 			CA:                 "myCa",
-			CAOptional:         true,
 			Cert:               "mycert.pem",
 			Key:                "mycert.key",
 			InsecureSkipVerify: true,
@@ -786,12 +782,11 @@ func TestDo_staticConfiguration(t *testing.T) {
 	}
 
 	config.Providers.HTTP = &http.Provider{
-		Endpoint:     "Myenpoint",
+		Endpoint:     "Myendpoint",
 		PollInterval: 42,
 		PollTimeout:  42,
 		TLS: &types.ClientTLS{
 			CA:                 "myCa",
-			CAOptional:         true,
 			Cert:               "mycert.pem",
 			Key:                "mycert.key",
 			InsecureSkipVerify: true,
@@ -804,38 +799,27 @@ func TestDo_staticConfiguration(t *testing.T) {
 		Debug:     true,
 	}
 
-	config.Metrics = &types.Metrics{
-		Prometheus: &types.Prometheus{
+	config.Metrics = &otypes.Metrics{
+		Prometheus: &otypes.Prometheus{
 			Buckets:              []float64{0.1, 0.3, 1.2, 5},
 			AddEntryPointsLabels: true,
 			AddServicesLabels:    true,
 			EntryPoint:           "MyEntryPoint",
 			ManualRouting:        true,
 		},
-		Datadog: &types.Datadog{
+		Datadog: &otypes.Datadog{
 			Address:              "localhost:8181",
 			PushInterval:         42,
 			AddEntryPointsLabels: true,
 			AddServicesLabels:    true,
 		},
-		StatsD: &types.Statsd{
+		StatsD: &otypes.Statsd{
 			Address:              "localhost:8182",
 			PushInterval:         42,
 			AddEntryPointsLabels: true,
 			AddServicesLabels:    true,
 			Prefix:               "MyPrefix",
 		},
-		InfluxDB: &types.InfluxDB{
-			Address:              "localhost:8183",
-			Protocol:             "http",
-			PushInterval:         42,
-			Database:             "myDB",
-			RetentionPolicy:      "12",
-			Username:             "a",
-			Password:             "aaaa",
-			AddEntryPointsLabels: true,
-			AddServicesLabels:    true,
-		},
 	}
 
 	config.Ping = &ping.Handler{
@@ -844,26 +828,49 @@ func TestDo_staticConfiguration(t *testing.T) {
 		TerminatingStatusCode: 42,
 	}
 
-	config.Log = &types.TraefikLog{
-		Level:    "Level",
-		FilePath: "/foo/path",
-		Format:   "json",
+	config.Log = &otypes.TraefikLog{
+		Level:      "Level",
+		Format:     "json",
+		FilePath:   "/foo/path",
+		MaxSize:    5,
+		MaxAge:     3,
+		MaxBackups: 4,
+		Compress:   true,
+		OTLP: &otypes.OTelLog{
+			ServiceName: "foobar",
+			ResourceAttributes: map[string]string{
+				"foobar": "foobar",
+			},
+			GRPC: &otypes.OTelGRPC{
+				Endpoint: "foobar",
+				Insecure: true,
+				Headers: map[string]string{
+					"foobar": "foobar",
+				},
+			},
+			HTTP: &otypes.OTelHTTP{
+				Endpoint: "foobar",
+				Headers: map[string]string{
+					"foobar": "foobar",
+				},
+			},
+		},
 	}
 
-	config.AccessLog = &types.AccessLog{
+	config.AccessLog = &otypes.AccessLog{
 		FilePath: "AccessLog FilePath",
 		Format:   "AccessLog Format",
-		Filters: &types.AccessLogFilters{
+		Filters: &otypes.AccessLogFilters{
 			StatusCodes:   []string{"200", "500"},
 			RetryAttempts: true,
 			MinDuration:   42,
 		},
-		Fields: &types.AccessLogFields{
+		Fields: &otypes.AccessLogFields{
 			DefaultMode: "drop",
 			Names: map[string]string{
 				"RequestHost": "keep",
 			},
-			Headers: &types.FieldHeaders{
+			Headers: &otypes.FieldHeaders{
 				DefaultMode: "drop",
 				Names: map[string]string{
 					"Referer": "keep",
@@ -871,61 +878,50 @@ func TestDo_staticConfiguration(t *testing.T) {
 			},
 		},
 		BufferingSize: 42,
+		OTLP: &otypes.OTelLog{
+			ServiceName: "foobar",
+			ResourceAttributes: map[string]string{
+				"foobar": "foobar",
+			},
+			GRPC: &otypes.OTelGRPC{
+				Endpoint: "foobar",
+				Insecure: true,
+				Headers: map[string]string{
+					"foobar": "foobar",
+				},
+			},
+			HTTP: &otypes.OTelHTTP{
+				Endpoint: "foobar",
+				Headers: map[string]string{
+					"foobar": "foobar",
+				},
+			},
+		},
 	}
 
 	config.Tracing = &static.Tracing{
-		ServiceName:   "myServiceName",
-		SpanNameLimit: 42,
-		Jaeger: &jaeger.Config{
-			SamplingServerURL:      "foobar",
-			SamplingType:           "foobar",
-			SamplingParam:          42,
-			LocalAgentHostPort:     "foobar",
-			Gen128Bit:              true,
-			Propagation:            "foobar",
-			TraceContextHeaderName: "foobar",
-			Collector: &jaeger.Collector{
+		ServiceName: "myServiceName",
+		ResourceAttributes: map[string]string{
+			"foobar": "foobar",
+		},
+		GlobalAttributes: map[string]string{
+			"foobar": "foobar",
+		},
+		SampleRate: 42,
+		OTLP: &otypes.OTelTracing{
+			HTTP: &otypes.OTelHTTP{
 				Endpoint: "foobar",
-				User:     "foobar",
-				Password: "foobar",
+				Headers: map[string]string{
+					"foobar": "foobar",
+				},
+			},
+			GRPC: &otypes.OTelGRPC{
+				Endpoint: "foobar",
+				Insecure: true,
+				Headers: map[string]string{
+					"foobar": "foobar",
+				},
 			},
-			DisableAttemptReconnecting: true,
-		},
-		Zipkin: &zipkin.Config{
-			HTTPEndpoint: "foobar",
-			SameSpan:     true,
-			ID128Bit:     true,
-			SampleRate:   42,
-		},
-		Datadog: &datadog.Config{
-			LocalAgentHostPort:         "foobar",
-			LocalAgentSocket:           "foobar",
-			GlobalTag:                  "foobar",
-			Debug:                      true,
-			PrioritySampling:           true,
-			TraceIDHeaderName:          "foobar",
-			ParentIDHeaderName:         "foobar",
-			SamplingPriorityHeaderName: "foobar",
-			BagagePrefixHeaderName:     "foobar",
-		},
-		Instana: &instana.Config{
-			LocalAgentHost: "foobar",
-			LocalAgentPort: 4242,
-			LogLevel:       "foobar",
-		},
-		Haystack: &haystack.Config{
-			LocalAgentHost:          "foobar",
-			LocalAgentPort:          42,
-			GlobalTag:               "foobar",
-			TraceIDHeaderName:       "foobar",
-			ParentIDHeaderName:      "foobar",
-			SpanIDHeaderName:        "foobar",
-			BaggagePrefixHeaderName: "foobar",
-		},
-		Elastic: &elastic.Config{
-			ServerURL:          "foobar",
-			SecretToken:        "foobar",
-			ServiceEnvironment: "foobar",
 		},
 	}
 
@@ -963,18 +959,34 @@ func TestDo_staticConfiguration(t *testing.T) {
 			"Descriptor0": {
 				ModuleName: "foobar",
 				Version:    "foobar",
+				Settings: plugins.Settings{
+					Envs:   []string{"a", "b"},
+					Mounts: []string{"a", "b"},
+				},
 			},
 			"Descriptor1": {
 				ModuleName: "foobar",
 				Version:    "foobar",
+				Settings: plugins.Settings{
+					Envs:   []string{"a", "b"},
+					Mounts: []string{"a", "b"},
+				},
 			},
 		},
 		LocalPlugins: map[string]plugins.LocalDescriptor{
 			"Descriptor0": {
 				ModuleName: "foobar",
+				Settings: plugins.Settings{
+					Envs:   []string{"a", "b"},
+					Mounts: []string{"a", "b"},
+				},
 			},
 			"Descriptor1": {
 				ModuleName: "foobar",
+				Settings: plugins.Settings{
+					Envs:   []string{"a", "b"},
+					Mounts: []string{"a", "b"},
+				},
 			},
 		},
 	}
@@ -990,17 +1002,7 @@ func TestDo_staticConfiguration(t *testing.T) {
 	}
 
 	expected := strings.TrimSuffix(string(expectedConfiguration), "\n")
-	assert.Equal(t, expected, cleanJSON)
+	assert.JSONEq(t, expected, cleanJSON)
 }
 
-func boolPtr(value bool) *bool {
-	return &value
-}
-
-func intPtr(value int) *int {
-	return &value
-}
-
-func int64Ptr(value int64) *int64 {
-	return &value
-}
+func pointer[T any](v T) *T { return &v }
